@@ -18,7 +18,6 @@ EXCLUDE = (
     "cmd/dot/dot.c",
     "cmd/dot/dot_builtins.cpp",
     "cmd/edgepaint/edgepaintmain.c",
-    "cmd/gvedit/csettings.cpp",
     "cmd/gvedit/csettings.h",
     "cmd/gvedit/imageviewer.cpp",
     "cmd/gvedit/imageviewer.h",
@@ -71,7 +70,6 @@ EXCLUDE = (
     "cmd/smyrna/hier.h",
     "cmd/smyrna/hotkeymap.c",
     "cmd/smyrna/hotkeymap.h",
-    "cmd/smyrna/main.c",
     "cmd/smyrna/polytess.c",
     "cmd/smyrna/polytess.h",
     "cmd/smyrna/selectionfuncs.c",
@@ -130,12 +128,6 @@ EXCLUDE = (
     "lib/ast/error.h",
     "lib/ast/fmtbuf.c",
     "lib/ast/fmtesc.c",
-    "lib/ast/pathaccess.c",
-    "lib/ast/pathcanon.c",
-    "lib/ast/pathcat.c",
-    "lib/ast/pathfind.c",
-    "lib/ast/pathpath.c",
-    "lib/ast/strcopy.c",
     "lib/ast/stresc.c",
     "lib/ast/strmatch.c",
     "lib/cdt/cdt.h",
@@ -165,6 +157,8 @@ EXCLUDE = (
     "lib/cgraph/graph.c",
     "lib/cgraph/id.c",
     "lib/cgraph/imap.c",
+    "lib/cgraph/ingraphs.c",
+    "lib/cgraph/ingraphs.h",
     "lib/cgraph/io.c",
     "lib/cgraph/mem.c",
     "lib/cgraph/node.c",
@@ -236,7 +230,6 @@ EXCLUDE = (
     "lib/common/utils.c",
     "lib/dotgen/acyclic.c",
     "lib/dotgen/aspect.c",
-    "lib/dotgen/aspect.h",
     "lib/dotgen/class1.c",
     "lib/dotgen/class2.c",
     "lib/dotgen/cluster.c",
@@ -254,7 +247,6 @@ EXCLUDE = (
     "lib/dotgen/rank.c",
     "lib/dotgen/sameport.c",
     "lib/edgepaint/edge_distinct_coloring.c",
-    "lib/edgepaint/edge_distinct_coloring.h",
     "lib/edgepaint/furtherest_point.c",
     "lib/edgepaint/furtherest_point.h",
     "lib/edgepaint/intersection.c",
@@ -262,9 +254,7 @@ EXCLUDE = (
     "lib/edgepaint/lab.c",
     "lib/edgepaint/lab.h",
     "lib/edgepaint/lab_gamut.c",
-    "lib/edgepaint/lab_gamut.h",
     "lib/edgepaint/node_distinct_coloring.c",
-    "lib/edgepaint/node_distinct_coloring.h",
     "lib/expr/excc.c",
     "lib/expr/excontext.c",
     "lib/expr/exdata.c",
@@ -344,7 +334,6 @@ EXCLUDE = (
     "lib/gvc/gvtextlayout.c",
     "lib/gvc/gvtool_tred.c",
     "lib/gvc/gvusershape.c",
-    "lib/gvpr/actions.c",
     "lib/gvpr/actions.h",
     "lib/gvpr/compile.c",
     "lib/gvpr/compile.h",
@@ -359,13 +348,10 @@ EXCLUDE = (
     "lib/gvpr/queue.h",
     "lib/gvpr/trie.c",
     "lib/gvpr/trieFA.h",
-    "lib/ingraphs/ingraphs.c",
-    "lib/ingraphs/ingraphs.h",
     "lib/label/index.c",
     "lib/label/index.h",
     "lib/label/node.c",
     "lib/label/node.h",
-    "lib/label/nrtmain.c",
     "lib/label/rectangle.c",
     "lib/label/rectangle.h",
     "lib/label/split.q.c",
@@ -373,7 +359,6 @@ EXCLUDE = (
     "lib/label/xlabels.c",
     "lib/label/xlabels.h",
     "lib/mingle/agglomerative_bundling.cpp",
-    "lib/mingle/agglomerative_bundling.h",
     "lib/mingle/edge_bundling.cpp",
     "lib/mingle/edge_bundling.h",
     "lib/mingle/ink.cpp",
@@ -489,7 +474,6 @@ EXCLUDE = (
     "lib/pathplan/shortestpth.c",
     "lib/pathplan/solvers.c",
     "lib/pathplan/solvers.h",
-    "lib/pathplan/tri.h",
     "lib/pathplan/triang.c",
     "lib/pathplan/util.c",
     "lib/pathplan/vis.h",
@@ -499,14 +483,12 @@ EXCLUDE = (
     "lib/rbtree/red_black_tree.h",
     "lib/rbtree/test_red_black_tree.c",
     "lib/sfdpgen/Multilevel.c",
-    "lib/sfdpgen/Multilevel.h",
     "lib/sfdpgen/post_process.c",
     "lib/sfdpgen/post_process.h",
     "lib/sfdpgen/sfdp.h",
     "lib/sfdpgen/sfdpinit.c",
     "lib/sfdpgen/sfdpinternal.h",
     "lib/sfdpgen/sparse_solve.c",
-    "lib/sfdpgen/sparse_solve.h",
     "lib/sfdpgen/spring_electrical.c",
     "lib/sfdpgen/spring_electrical.h",
     "lib/sfdpgen/stress_model.c",
@@ -519,8 +501,6 @@ EXCLUDE = (
     "lib/sfio/sftable.c",
     "lib/sfio/sfprint.c",
     "lib/sfio/sfvscanf.c",
-    "lib/sparse/BinaryHeap.c",
-    "lib/sparse/BinaryHeap.h",
     "lib/sparse/DotIO.c",
     "lib/sparse/DotIO.h",
     "lib/sparse/LinkedList.c",
@@ -609,8 +589,6 @@ EXCLUDE = (
     "plugin/gd/gvrender_gd.c",
     "plugin/gd/gvrender_gd_vrml.c",
     "plugin/gd/gvtextlayout_gd.c",
-    "plugin/gdiplus/FileStream.cpp",
-    "plugin/gdiplus/FileStream.h",
     "plugin/gdiplus/gvdevice_gdiplus.cpp",
     "plugin/gdiplus/gvloadimage_gdiplus.cpp",
     "plugin/gdiplus/gvplugin_gdiplus.cpp",
@@ -658,7 +636,6 @@ EXCLUDE = (
     "plugin/xlib/gvdevice_xlib.c",
     "plugin/xlib/gvplugin_xlib.c",
     "tclpkg/gdtclft/gdtclft.c",
-    "tclpkg/gv/gv.cpp",
     "tclpkg/gv/gv_builtins.c",
     "tclpkg/gv/gv_dummy_init.c",
     "tclpkg/gv/gv_java_init.c",
@@ -671,11 +648,8 @@ EXCLUDE = (
     "tclpkg/tcldot/tcldot-io.c",
     "tclpkg/tcldot/tcldot-nodecmd.c",
     "tclpkg/tcldot/tcldot-util.c",
-    "tclpkg/tcldot/tcldot.c",
     "tclpkg/tcldot/tcldot.h",
     "tclpkg/tcldot/tcldot_builtins.c",
-    "tclpkg/tclhandle/tclhandle.c",
-    "tclpkg/tclhandle/tclhandle.h",
     "tclpkg/tclpathplan/find_ints.c",
     "tclpkg/tclpathplan/intersect.c",
     "tclpkg/tclpathplan/makecw.c",
@@ -684,8 +658,6 @@ EXCLUDE = (
     "tclpkg/tclpathplan/wrapper.c",
     "tclpkg/tclstubs/tclInt.h",
     "tclpkg/tclstubs/tclStubLib.c",
-    "tests/check-package-version.c",
-    "tests/unit_tests/lib/common/command_line.c",
     "windows/gvedit/Application.h",
     "windows/gvedit/GraphX.cpp",
     "windows/gvedit/UAbout.cpp",
