@@ -16,6 +16,7 @@ from gvtest import (  # pylint: disable=wrong-import-position
     dot,
     freedesktop_os_release,
     is_cmake,
+    is_macos,
     is_mingw,
     which,
 )
@@ -93,14 +94,26 @@ def test_existence(binary: str):
         "vimdot",
     ]
 
+    tools_not_built_with_cmake = [
+        "dot_builtins",
+    ]
+
     os_id = freedesktop_os_release().get("ID")
 
     # FIXME: Remove skip when
     # https://gitlab.com/graphviz/graphviz/-/issues/1834 is fixed
-    if os_id == "centos" and binary == "smyrna":
+    if os_id in ("centos", "rocky") and binary == "smyrna":
         check_that_tool_does_not_exist(binary, os_id)
         pytest.skip("smyrna is not built for Centos (#1834)")
 
+    if os_id == "rocky" and binary == "mingle":
+        check_that_tool_does_not_exist(binary, os_id)
+        pytest.skip("mingle is not built for Rocky due to lacking libANN")
+
+    if is_cmake() and binary in tools_not_built_with_cmake:
+        check_that_tool_does_not_exist(binary, os_id)
+        pytest.skip(f"{binary} is not built with CMake")
+
     # FIXME: Remove skip when
     # https://gitlab.com/graphviz/graphviz/-/issues/1837 is fixed
     if os.getenv("build_system") == "msbuild":
@@ -125,10 +138,22 @@ def test_existence(binary: str):
         check_that_tool_does_not_exist(binary, os_id)
         pytest.skip("smyrna is not built on non-Linux due to lacking dependencies")
 
-    if binary in ("gvmap.sh", "vimdot") and platform.system() == "Windows":
+    if binary == "gvmap.sh" and is_mingw():
+        check_that_tool_does_not_exist(binary, os_id)
+        pytest.skip(f"{binary} is not installed on MinGW")
+
+    if binary == "vimdot" and platform.system() == "Windows":
         check_that_tool_does_not_exist(binary, os_id)
         pytest.skip(f"{binary} is not installed on Windows")
 
+    if binary == "smyrna" and not is_cmake() and is_macos():
+        check_that_tool_does_not_exist(binary, os_id)
+        pytest.skip("https://gitlab.com/graphviz/graphviz/-/issues/2422")
+
+    if binary == "vimdot" and not is_cmake() and is_macos():
+        check_that_tool_does_not_exist(binary, os_id)
+        pytest.skip("https://gitlab.com/graphviz/graphviz/-/issues/2423")
+
     assert which(binary) is not None
 
 
@@ -143,7 +168,7 @@ def check_that_tool_does_not_exist(tool, os_id):
 
 
 @pytest.mark.xfail(
-    is_cmake() and platform.system() == "Windows" and not is_mingw(),
+    is_cmake() and not is_mingw() and is_win64(),
     reason="png:gd unavailable when built with CMake",
     strict=True,
 )  # FIXME
@@ -157,6 +182,16 @@ def test_1786():
     dot("png:gd", source="digraph { a -> b; }")
 
 
+def test_2469():
+    """
+    svgz format should be supported
+    https://gitlab.com/graphviz/graphviz/-/issues/2469
+    """
+
+    # run a trivial graph through Graphviz
+    dot("svgz", source="digraph { a -> b; }")
+
+
 def test_installation():
     """
     check that Graphviz reports the expected version number
