@@ -23,14 +23,15 @@
 #include "config.h"
 
 #include <getopt.h>
-
+#include <algorithm>
 #include <assert.h>
 #include <gvc/gvc.h>
 #include <cgraph/alloc.h>
 #include <cgraph/exit.h>
+#include <cgraph/ingraphs.h>
+#include <cgraph/startswith.h>
 #include <common/render.h>
 #include <neatogen/neatoprocs.h>
-#include <ingraphs/ingraphs.h>
 #include <iostream>
 #include <limits>
 #include <map>
@@ -474,7 +475,7 @@ static std::string xName(used_t &names, char *oldname) {
 
 #define MARK(e) (ED_alg(e) = e)
 #define MARKED(e) (ED_alg(e))
-#define ISCLUSTER(g) (!strncmp(agnameof(g),"cluster",7))
+#define ISCLUSTER(g) startswith(agnameof(g), "cluster")
 #define SETCLUST(g,h) (GD_alg(g) = h)
 #define GETCLUST(g) ((Agraph_t*)GD_alg(g))
 
@@ -692,10 +693,10 @@ static boxf compBB(std::vector<Agraph_t*> &gs) {
 
     for (size_t i = 1; i < gs.size(); i++) {
 	bb2 = GD_bb(gs[i]);
-	bb.LL.x = MIN(bb.LL.x, bb2.LL.x);
-	bb.LL.y = MIN(bb.LL.y, bb2.LL.y);
-	bb.UR.x = MAX(bb.UR.x, bb2.UR.x);
-	bb.UR.y = MAX(bb.UR.y, bb2.UR.y);
+	bb.LL.x = std::min(bb.LL.x, bb2.LL.x);
+	bb.LL.y = std::min(bb.LL.y, bb2.LL.y);
+	bb.UR.x = std::max(bb.UR.x, bb2.UR.x);
+	bb.UR.y = std::max(bb.UR.y, bb2.UR.y);
     }
 
     return bb;
