@@ -1,4 +1,9 @@
-/*************************************************************************
+/**
+ * @file
+ * @brief implementation of dynamic attributes
+ * @ingroup cgraph_attr
+ */
+ /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
  * are made available under the terms of the Eclipse Public License v1.0
@@ -9,6 +14,7 @@
  *************************************************************************/
 
 #include	<cgraph/cghdr.h>
+#include	<cgraph/streq.h>
 #include	<cgraph/unreachable.h>
 #include	<stddef.h>
 #include	<stdbool.h>
@@ -30,7 +36,6 @@ Dtdisc_t AgDataDictDisc = {
     NULL,
     freesym,
     NULL,
-    NULL,
 };
 
 static char DataDictName[] = "_AG_datadict";
@@ -303,8 +308,7 @@ static Agsym_t *setattr(Agraph_t * g, int kind, char *name, const char *value) {
 	    dtinsert(rdict, rsym);
 	    switch (kind) {
 	    case AGRAPH:
-		agapply(root, (Agobj_t *) root, (agobjfn_t) addattr,
-			rsym, TRUE);
+		agapply(root, (Agobj_t *)root, (agobjfn_t)addattr, rsym, true);
 		break;
 	    case AGNODE:
 		for (n = agfstnode(root); n; n = agnxtnode(root, n))
@@ -532,7 +536,7 @@ static void init_all_attrs(Agraph_t * g)
     Agedge_t *e;
 
     root = agroot(g);
-    agapply(root, (Agobj_t*)root, agraphattr_init_wrapper, NULL, TRUE);
+    agapply(root, (Agobj_t*)root, agraphattr_init_wrapper, NULL, true);
     for (n = agfstnode(root); n; n = agnxtnode(root, n)) {
 	agnodeattr_init(g, n);
 	for (e = agfstout(root, n); e; e = agnxtout(root, e)) {
