@@ -1,3 +1,7 @@
+/// @file
+/// @brief generic manipulations with @ref Agobj_t and derived objects graphs, nodes and edges
+/// @ingroup cgraph_object
+
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -9,6 +13,7 @@
  *************************************************************************/
 
 #include <cgraph/cghdr.h>
+#include <cgraph/unreachable.h>
 #include <stddef.h>
 
 int agdelete(Agraph_t * g, void *obj)
@@ -42,13 +47,11 @@ int agrename(Agobj_t * obj, char *newname)
 	old_id = AGID(obj);
 	g = agraphof(obj);
 	/* can we reserve the id corresponding to newname? */
-	if (agmapnametoid(agroot(g), AGTYPE(obj), newname,
-			  &new_id, FALSE) == 0)
+	if (agmapnametoid(agroot(g), AGTYPE(obj), newname, &new_id, false) == 0)
 	    return FAILURE;
 	if (new_id == old_id)
 	    return SUCCESS;
-	if (agmapnametoid(agroot(g), AGTYPE(obj), newname,
-			  &new_id, TRUE) == 0)
+	if (agmapnametoid(agroot(g), AGTYPE(obj), newname, &new_id, true) == 0)
 	    return FAILURE;
         /* obj* is unchanged, so no need to re agregister() */
 	if (agparent(g) && agidsubg(agparent(g), new_id, 0))
@@ -58,11 +61,11 @@ int agrename(Agobj_t * obj, char *newname)
 	break;
     case AGNODE:
 	return agrelabel_node((Agnode_t *) obj, newname);
-	agrename(obj, newname);
-	break;
     case AGINEDGE:
     case AGOUTEDGE:
 	return FAILURE;
+    default:
+	UNREACHABLE();
     }
     return SUCCESS;
 }
@@ -73,10 +76,7 @@ int agrename(Agobj_t * obj, char *newname)
 
 void agmethod_init(Agraph_t * g, void *obj)
 {
-    if (g->clos->callbacks_enabled)
-	aginitcb(g, obj, g->clos->cb);
-    else
-	agrecord_callback(g, obj, CB_INITIALIZE, NULL);
+  aginitcb(g, obj, g->clos->cb);
 }
 
 void aginitcb(Agraph_t * g, void *obj, Agcbstack_t * cbstack)
@@ -104,10 +104,7 @@ void aginitcb(Agraph_t * g, void *obj, Agcbstack_t * cbstack)
 
 void agmethod_upd(Agraph_t * g, void *obj, Agsym_t * sym)
 {
-    if (g->clos->callbacks_enabled)
-	agupdcb(g, obj, sym, g->clos->cb);
-    else
-	agrecord_callback(g, obj, CB_UPDATE, sym);
+  agupdcb(g, obj, sym, g->clos->cb);
 }
 
 void agupdcb(Agraph_t * g, void *obj, Agsym_t * sym, Agcbstack_t * cbstack)
@@ -135,10 +132,7 @@ void agupdcb(Agraph_t * g, void *obj, Agsym_t * sym, Agcbstack_t * cbstack)
 
 void agmethod_delete(Agraph_t * g, void *obj)
 {
-    if (g->clos->callbacks_enabled)
-	agdelcb(g, obj, g->clos->cb);
-    else
-	agrecord_callback(g, obj, CB_DELETION, NULL);
+  agdelcb(g, obj, g->clos->cb);
 }
 
 void agdelcb(Agraph_t * g, void *obj, Agcbstack_t * cbstack)
