@@ -8,13 +8,14 @@
  * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
+#include <assert.h>
 #include <limits.h>
 #include <stdbool.h>
 #include <stddef.h>
 #include <stdio.h>		/* need sprintf() */
 #include <ctype.h>
 #include <cgraph/cghdr.h>
-#include <cgraph/likely.h>
+#include <cgraph/gv_ctype.h>
 #include <cgraph/strcasecmp.h>
 #include <inttypes.h>
 
@@ -47,7 +48,44 @@ static int indent(Agraph_t * g, iochan_t * ofile)
 
 // alphanumeric, '.', '-', or non-ascii; basically, chars used in unquoted ids
 static bool is_id_char(char c) {
-  return isalnum(c) || c == '.' || c == '-' || !isascii(c);
+  return gv_isalnum(c) || c == '.' || c == '-' || !isascii(c);
+}
+
+// is the prefix of this string a recognized Graphviz escape sequence?
+// https://graphviz.org/docs/attr-types/escString/
+static bool is_escape(const char *str) {
+  assert(str != NULL);
+
+  if (*str != '\\')
+    return false;
+
+  if (str[1] == 'E')
+    return true;
+  if (str[1] == 'G')
+    return true;
+  if (str[1] == 'H')
+    return true;
+  if (str[1] == 'L')
+    return true;
+  if (str[1] == 'N')
+    return true;
+  if (str[1] == 'T')
+    return true;
+
+  if (str[1] == 'l')
+    return true;
+  if (str[1] == 'n')
+    return true;
+  if (str[1] == 'r')
+    return true;
+
+  if (str[1] == '\\')
+    return true;
+
+  if (str[1] == '"')
+    return true;
+
+  return false;
 }
 
 /* _agstrcanon:
@@ -60,6 +98,7 @@ static char *_agstrcanon(char *arg, char *buf)
     char uc;
     int cnt = 0, dotcnt = 0;
     bool needs_quotes = false;
+    bool part_of_escape = false;
     bool maybe_num;
     bool backslash_pending = false;
     static const char *tokenlist[]	/* must agree with scan.l */
@@ -74,13 +113,15 @@ static char *_agstrcanon(char *arg, char *buf)
     p = buf;
     *p++ = '\"';
     uc = *s++;
-    maybe_num = isdigit(uc) != 0 || uc == '.' || uc == '-';
+    maybe_num = gv_isdigit(uc) || uc == '.' || uc == '-';
     while (uc) {
-	if (uc == '\"') {
+	if (uc == '\"' && !part_of_escape) {
 	    *p++ = '\\';
 	    needs_quotes = true;
-	} 
-	else if (maybe_num) {
+	} else if (!part_of_escape && is_escape(&s[-1])) {
+	    needs_quotes = true;
+	    part_of_escape = true;
+	} else if (maybe_num) {
 	    if (uc == '-') {
 		if (cnt) {
 		    maybe_num = false;
@@ -93,13 +134,16 @@ static char *_agstrcanon(char *arg, char *buf)
 		    needs_quotes = true;
 		}
 	    }
-	    else if (!isdigit(uc)) {
+	    else if (!gv_isdigit(uc)) {
 		maybe_num = false;
 		needs_quotes = true;
 	    }
+	    part_of_escape = false;
 	}
-	else if (!(isalnum(uc) || uc == '_' || !isascii(uc)))
+	else if (!(gv_isalnum(uc) || uc == '_' || !isascii(uc))) {
 	    needs_quotes = true;
+	    part_of_escape = false;
+	}
 	*p++ = uc;
 	uc = *s++;
 	cnt++;
@@ -170,7 +214,7 @@ static char *getoutputbuffer(const char *str)
     req = MAX(2 * strlen(str) + 2, BUFSIZ);
     if (req > len) {
 	char *r = realloc(rv, req);
-	if (UNLIKELY(r == NULL))
+	if (r == NULL)
 	    return NULL;
 	rv = r;
 	len = req;
@@ -186,7 +230,7 @@ static char *getoutputbuffer(const char *str)
 char *agcanonStr(char *str)
 {
     char *buffer = getoutputbuffer(str);
-    if (UNLIKELY(buffer == NULL))
+    if (buffer == NULL)
         return NULL;
     return agstrcanon(str, buffer);
 }
@@ -199,7 +243,7 @@ char *agcanonStr(char *str)
 char *agcanon(char *str, int html)
 {
     char* buf = getoutputbuffer(str);
-    if (UNLIKELY(buf == NULL))
+    if (buf == NULL)
 	return NULL;
     if (html)
 	return agcanonhtmlstr(str, buf);
@@ -207,14 +251,12 @@ char *agcanon(char *str, int html)
 	return _agstrcanon(str, buf);
 }
 
-static int _write_canonstr(Agraph_t * g, iochan_t * ofile, char *str,
-			   int chk)
-{
+static int _write_canonstr(Agraph_t *g, iochan_t *ofile, char *str, bool chk) {
     if (chk) {
 	str = agcanonStr(str);
     } else {
 	char *buffer = getoutputbuffer(str);
-	if (UNLIKELY(buffer == NULL))
+	if (buffer == NULL)
 	    return EOF;
 	str = _agstrcanon(str, buffer);
     }
@@ -224,22 +266,20 @@ static int _write_canonstr(Agraph_t * g, iochan_t * ofile, char *str,
 static int write_canonstr(Agraph_t * g, iochan_t * ofile, char *str)
 {
     char *s;
-    int r;
 
     /* str may not have been allocated by agstrdup, so we first need to turn it
      * into a valid refstr
      */
     s = agstrdup(g, str);
 
-    r = _write_canonstr(g, ofile, s, TRUE);
+    int r = _write_canonstr(g, ofile, s, true);
 
     agstrfree(g, s);
     return r;
 }
 
 static int write_dict(Agraph_t * g, iochan_t * ofile, char *name,
-		      Dict_t * dict, int top)
-{
+                      Dict_t * dict, bool top) {
     int cnt = 0;
     Dict_t *view;
     Agsym_t *sym, *psym;
@@ -283,10 +323,9 @@ static int write_dict(Agraph_t * g, iochan_t * ofile, char *name,
     return 0;
 }
 
-static int write_dicts(Agraph_t * g, iochan_t * ofile, int top)
-{
+static int write_dicts(Agraph_t *g, iochan_t *ofile, bool top) {
     Agdatadict_t *def;
-    if ((def = agdatadict(g, FALSE))) {
+    if ((def = agdatadict(g, false))) {
 	CHKRV(write_dict(g, ofile, "graph", def->dict.g, top));
 	CHKRV(write_dict(g, ofile, "node", def->dict.n, top));
 	CHKRV(write_dict(g, ofile, "edge", def->dict.e, top));
@@ -294,8 +333,7 @@ static int write_dicts(Agraph_t * g, iochan_t * ofile, int top)
     return 0;
 }
 
-static int write_hdr(Agraph_t * g, iochan_t * ofile, int top)
-{
+static int write_hdr(Agraph_t *g, iochan_t *ofile, bool top) {
     char *name, *sep, *kind, *strict;
     bool root = false;
     bool hasName = true;
@@ -334,7 +372,7 @@ static int write_hdr(Agraph_t * g, iochan_t * ofile, int top)
     CHKRV(ioput(g, ofile, "{\n"));
     Level++;
     CHKRV(write_dicts(g, ofile, top));
-    AGATTRWF(g) = TRUE;
+    AGATTRWF(g) = true;
     return 0;
 }
 
@@ -366,7 +404,7 @@ static bool irrelevant_subgraph(Agraph_t * g)
 		&& strcmp(sdata->str[i], pdata->str[i]))
 		return false;
     }
-    dd = agdatadict(g, FALSE);
+    dd = agdatadict(g, false);
     if (!dd)
 	return true;
     if (dtsize(dd->dict.n) > 0 || dtsize(dd->dict.e) > 0)
@@ -381,7 +419,7 @@ static bool node_in_subg(Agraph_t * g, Agnode_t * n)
     for (subg = agfstsubg(g); subg; subg = agnxtsubg(subg)) {
 	if (irrelevant_subgraph(subg))
 	    continue;
-	if (agsubnode(subg, n, FALSE))
+	if (agsubnode(subg, n, 0))
 	    return true;
     }
     return false;
@@ -429,7 +467,7 @@ static int write_subgs(Agraph_t * g, iochan_t * ofile)
 	    write_subgs(subg, ofile);
 	}
 	else {
-	    CHKRV(write_hdr(subg, ofile, FALSE));
+	    CHKRV(write_hdr(subg, ofile, false));
 	    CHKRV(write_body(subg, ofile));
 	    CHKRV(write_trl(subg, ofile));
 	}
@@ -437,9 +475,7 @@ static int write_subgs(Agraph_t * g, iochan_t * ofile)
     return 0;
 }
 
-static int write_edge_name(Agedge_t * e, iochan_t * ofile, int terminate)
-{
-    int rv;
+static int write_edge_name(Agedge_t *e, iochan_t *ofile, bool terminate) {
     char *p;
     Agraph_t *g;
 
@@ -453,10 +489,9 @@ static int write_edge_name(Agedge_t * e, iochan_t * ofile, int terminate)
 	CHKRV(write_canonstr(g, ofile, p));
 	if (terminate)
 	    CHKRV(ioput(g, ofile, "]"));
-	rv = TRUE;
-    } else
-	rv = FALSE;
-    return rv;
+	return 1;
+    }
+    return 0;
 }
 
 
@@ -470,7 +505,7 @@ static int write_nondefault_attrs(void *obj, iochan_t * ofile,
     int rv;
 
     if (AGTYPE(obj) == AGINEDGE || AGTYPE(obj) == AGOUTEDGE) {
-	CHKRV(rv = write_edge_name(obj, ofile, FALSE));
+	CHKRV(rv = write_edge_name(obj, ofile, false));
 	if (rv)
 	    cnt++;
     }
@@ -501,7 +536,7 @@ static int write_nondefault_attrs(void *obj, iochan_t * ofile,
 	CHKRV(ioput(g, ofile, "]"));
 	Level--;
     }
-    AGATTRWF(obj) = TRUE;
+    AGATTRWF(obj) = true;
     return 0;
 }
 
@@ -572,29 +607,28 @@ static int write_port(Agedge_t * e, iochan_t * ofile, Agsym_t * port)
 	char *s = strchr(val, ':');
 	if (s) {
 	    *s = '\0';
-	    CHKRV(_write_canonstr(g, ofile, val, FALSE));
+	    CHKRV(_write_canonstr(g, ofile, val, false));
 	    CHKRV(ioput(g, ofile, ":"));
-	    CHKRV(_write_canonstr(g, ofile, s + 1, FALSE));
+	    CHKRV(_write_canonstr(g, ofile, s + 1, false));
 	    *s = ':';
 	} else {
-	    CHKRV(_write_canonstr(g, ofile, val, FALSE));
+	    CHKRV(_write_canonstr(g, ofile, val, false));
 	}
     }
     return 0;
 }
 
-static int write_edge_test(Agraph_t * g, Agedge_t * e)
-{
+static bool write_edge_test(Agraph_t *g, Agedge_t *e) {
     Agraph_t *subg;
 
     /* can use agedge() because we subverted the dict compar_f */
     for (subg = agfstsubg(g); subg; subg = agnxtsubg(subg)) {
 	if (irrelevant_subgraph(subg))
 	    continue;
-	if (agsubedge(subg, e, FALSE))
-	    return FALSE;
+	if (agsubedge(subg, e, 0))
+	    return false;
     }
-    return TRUE;
+    return true;
 }
 
 static int write_edge(Agedge_t * e, iochan_t * ofile, Dict_t * d)
@@ -614,7 +648,7 @@ static int write_edge(Agedge_t * e, iochan_t * ofile, Dict_t * d)
     if (!attrs_written(e)) {
 	CHKRV(write_nondefault_attrs(e, ofile, d));
     } else {
-	CHKRV(write_edge_name(e, ofile, TRUE));
+	CHKRV(write_edge_name(e, ofile, true));
     }
     return ioput(g, ofile, ";\n");
 }
@@ -626,7 +660,7 @@ static int write_body(Agraph_t * g, iochan_t * ofile)
     Agdatadict_t *dd;
 
     CHKRV(write_subgs(g, ofile));
-    dd = agdatadict(g, FALSE);
+    dd = agdatadict(g, false);
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
 	if (write_node_test(g, n, AGSEQ(n)))
 	    CHKRV(write_node(n, ofile, dd ? dd->dict.n : 0));
@@ -671,13 +705,13 @@ int agwrite(Agraph_t * g, void *ofile)
     char* s;
     Level = 0;			/* re-initialize tab level */
     s = agget(g, "linelength");
-    if (s != NULL && isdigit((int)*s)) {
+    if (s != NULL && gv_isdigit(*s)) {
 	unsigned long len = strtoul(s, NULL, 10);
 	if ((len == 0 || len >= MIN_OUTPUTLINE) && len <= (unsigned long)INT_MAX)
 	    Max_outputline = (int)len;
     }
     set_attrwf(g, true, false);
-    CHKRV(write_hdr(g, ofile, TRUE));
+    CHKRV(write_hdr(g, ofile, true));
     CHKRV(write_body(g, ofile));
     CHKRV(write_trl(g, ofile));
     Max_outputline = MAX_OUTPUTLINE;
