@@ -17,20 +17,19 @@
  * Needs to be fixed before layout engines can be plugins.
  */
 
-#include <cgraph/likely.h>
-#include <ctype.h>
+#include <assert.h>
+#include <cgraph/gv_ctype.h>
 #include <common/render.h>
 #include <fdpgen/tlayout.h>
 #include <gvc/gvc.h>
 #include <fdpgen/fdp.h>
+#include <stdbool.h>
 
 /* neato_extra_args:
  * Handle special neato arguments.
  * Return number of unprocessed arguments; return < 0 on error.
  */
-static int
-neato_extra_args(GVC_t *gvc, int argc, char** argv)
-{
+static int neato_extra_args(int argc, char** argv) {
   char** p = argv+1;
   int    i;
   char*  arg;
@@ -38,9 +37,10 @@ neato_extra_args(GVC_t *gvc, int argc, char** argv)
 
   for (i = 1; i < argc; i++) {
     arg = argv[i];
-    if (arg && *arg == '-') {
+    assert(arg != NULL);
+    if (arg[0] == '-') {
       switch (arg[1]) {
-      case 'x' : Reduce = TRUE; break;
+      case 'x' : Reduce = true; break;
       case 'n':
         if (arg[2]) {
           Nop = atoi(arg+2);
@@ -69,50 +69,6 @@ neato_extra_args(GVC_t *gvc, int argc, char** argv)
   return cnt;
 }
 
-/* memtest_extra_args:
- * Handle special memtest arguments.
- * Return number of unprocessed arguments; return < 0 on error.
- */
-static int
-memtest_extra_args(GVC_t *gvc, int argc, char** argv)
-{
-  char** p = argv+1;
-  int    i;
-  char*  arg;
-  int    cnt = 1;
-
-  for (i = 1; i < argc; i++) {
-    arg = argv[i];
-    if (arg && *arg == '-') {
-      switch (arg[1]) {
-      case 'm' :
-        if (arg[2]) {
-          MemTest = atoi(arg+2);
-          if (MemTest <= 0) {
-            agerr (AGERR, "Invalid parameter \"%s\" for -m flag\n", arg+2);
-            dotneato_usage (1);
-	    return -1;
-          }
-        }
-        else MemTest = -1;
-	break;
-      default :
-        cnt++;
-        if (*p != arg) *p = arg;
-        p++;
-        break;
-      }
-    }
-    else {
-      cnt++;
-      if (*p != arg) *p = arg;
-      p++;
-    }
-  }
-  *p = 0;
-  return cnt;
-}
-
 /* config_extra_args:
  * Handle special config arguments.
  * Return number of unprocessed arguments; return < 0 on error.
@@ -127,18 +83,19 @@ config_extra_args(GVC_t *gvc, int argc, char** argv)
 
   for (i = 1; i < argc; i++) {
     arg = argv[i];
-    if (arg && *arg == '-') {
+    assert(arg != NULL);
+    if (arg[0] == '-') {
       switch (arg[1]) {
       case 'v':
 	gvc->common.verbose = 1;
-	if (isdigit(arg[2]))
+	if (gv_isdigit(arg[2]))
 	  gvc->common.verbose = atoi(&arg[2]);
         break;
       case 'O' :
-          gvc->common.auto_outfile_names = TRUE;
+          gvc->common.auto_outfile_names = true;
 	  break;
       case 'c' :
-          gvc->common.config = TRUE;
+          gvc->common.config = true;
 	  break;
       default :
         cnt++;
@@ -237,9 +194,7 @@ setFDPAttr (char* arg)
  * These have the form -L<name>=<value>.
  * Return number of unprocessed arguments; return < 0 on error.
  */
-static int
-fdp_extra_args (GVC_t *gvc, int argc, char** argv)
-{
+static int fdp_extra_args(int argc, char** argv) {
   char** p = argv+1;
   int    i;
   char*  arg;
@@ -247,7 +202,8 @@ fdp_extra_args (GVC_t *gvc, int argc, char** argv)
 
   for (i = 1; i < argc; i++) {
     arg = argv[i];
-    if (arg && (*arg == '-') && (*(arg+1) == 'L')) {
+    assert(arg != NULL);
+    if (arg[0] == '-' && arg[1] == 'L') {
       if (setFDPAttr (arg+2)) {
 	dotneato_usage(1);
 	return -1;
@@ -270,17 +226,15 @@ fdp_extra_args (GVC_t *gvc, int argc, char** argv)
 int gvParseArgs(GVC_t *gvc, int argc, char** argv)
 {
     int rv;
-    if ((argc = neato_extra_args(gvc, argc, argv)) < 0)
+    if ((argc = neato_extra_args(argc, argv)) < 0)
 	return (1-argc);
-    if ((argc = fdp_extra_args(gvc, argc, argv)) < 0)
-	return (1-argc);
-    if ((argc = memtest_extra_args(gvc, argc, argv)) < 0)
+    if ((argc = fdp_extra_args(argc, argv)) < 0)
 	return (1-argc);
     if ((argc = config_extra_args(gvc, argc, argv)) < 0)
 	return (1-argc);
     if ((rv = dotneato_args_initialize(gvc, argc, argv)))
 	return rv;
-    if (UNLIKELY(Verbose))
+    if (Verbose)
 	gvplugin_write_status(gvc);
     return 0;
 }
