@@ -213,7 +213,7 @@ int compoundEdges(graph_t * g, expand_t* pm, int edgetype)
     node_t *head;
     edge_t *e;
     edge_t *e0;
-    vconfig_t *vconfig;
+    vconfig_t *vconfig = NULL;
     int rv = 0;
 
     for (n = agfstnode(g); n; n = agnxtnode(g, n)) {
@@ -254,7 +254,7 @@ int compoundEdges(graph_t * g, expand_t* pm, int edgetype)
 		 * with change in direction, different arrowheads, labels, etc.
 		 */
 		for (e0 = e; e0; e0 = ED_to_virt(e0)) {
-		    ED_path(e0) = getPath(e0, vconfig, 0);
+		    ED_path(e0) = getPath(e0, vconfig, false);
 		    assert(objlist_size(&objl) <= INT_MAX);
 		    makeSpline(e0, objlist_at(&objl, 0), (int)objlist_size(&objl), false);
 		}
@@ -262,5 +262,8 @@ int compoundEdges(graph_t * g, expand_t* pm, int edgetype)
 	    }
 	}
     }
+    if (vconfig != NULL) {
+	Pobsclose(vconfig);
+    }
     return rv;
 }
