@@ -14,14 +14,17 @@
 #define _GNU_SOURCE 1
 #endif
 
+#include <assert.h>
+#include <cgraph/agxbuf.h>
 #include <cgraph/alloc.h>
 #include <cgraph/exit.h>
+#include <cgraph/gv_ctype.h>
+#include <cgraph/startswith.h>
 #include <gvc/gvconfig.h>
-
-#include <ctype.h>
 #include <stdbool.h>
-#include <stddef.h>
+#include <stdlib.h>
 #include	<string.h>
+#include <unistd.h>
 
 #ifdef ENABLE_LTDL
 #ifdef HAVE_DL_ITERATE_PHDR
@@ -60,7 +63,7 @@ static int glob (GVC_t * gvc, char*, int, int (*errfunc)(const char *, int), glo
 #include        <gvc/gvcproc.h>
 
 /* FIXME */
-extern Dt_t * textfont_dict_open(GVC_t *gvc);
+extern void textfont_dict_open(GVC_t *gvc);
 
 /*
     A config for gvrender is a text file containing a
@@ -170,9 +173,8 @@ static char *token(int *nest, char **tokens)
 
 static int gvconfig_plugin_install_from_config(GVC_t * gvc, char *s)
 {
-    char *package_path, *name, *api;
+    char *package_path, *name;
     const char *type;
-    api_t gv_api;
     int quality;
     int nest = 0;
     gvplugin_package_t *package;
@@ -186,8 +188,12 @@ static int gvconfig_plugin_install_from_config(GVC_t * gvc, char *s)
 	    name = "x";
         package = gvplugin_package_record(gvc, package_path, name);
 	do {
-	    api = token(&nest, &s);
-	    gv_api = gvplugin_api(api);
+	    const char *api = token(&nest, &s);
+	    const api_t gv_api = gvplugin_api(api);
+	    if (gv_api == (api_t)-1) {
+		agerr(AGERR, "config error: %s %s not found\n", package_path, api);
+		return 0;
+	    }
 	    do {
 		if (nest == 2) {
 		    type = token(&nest, &s);
@@ -260,7 +266,6 @@ static void gvconfig_write_library_config(GVC_t *gvc, char *lib_path,
 
 #define BSZ 1024
 #define DOTLIBS "/.libs"
-#define STRLEN(s) (sizeof(s)-1)
 
 #ifdef HAVE_DL_ITERATE_PHDR
 static int line_callback(struct dl_phdr_info *info, size_t size, void *line)
@@ -325,7 +330,7 @@ char * gvconfig_libdir(GVC_t * gvc)
 			const char *s = tmp - 1;
 			/* back up to previous slash (or head of string) */
 			while (*s != '/' && s > p) s--;
-			if (strncmp (s, DOTLIBS, STRLEN(DOTLIBS)) == 0)
+			if (startswith(s, DOTLIBS))
 			    continue;
 		    }
 
@@ -421,18 +426,17 @@ static bool is_plugin(const char *filepath) {
 
     // does this filename contain the expected version?
     if (len < strlen(VERSION)
-        || strncmp(filepath + len - strlen(VERSION), VERSION,
-          strlen(VERSION)) != 0) {
+        || !startswith(filepath + len - strlen(VERSION), VERSION)) {
 	return false;
     }
     len -= strlen(VERSION);
 
 #else
     // does this filename have a version?
-    if (len == 0 || !isdigit((int)filepath[len - 1])) {
+    if (len == 0 || !gv_isdigit(filepath[len - 1])) {
 	return false;
     }
-    while (len > 0 && isdigit((int)filepath[len - 1])) {
+    while (len > 0 && gv_isdigit(filepath[len - 1])) {
 	--len;
     }
 
@@ -440,27 +444,25 @@ static bool is_plugin(const char *filepath) {
 
     // ensure the remainder conforms to what we expect of a shared library
 #if defined(DARWIN_DYLIB)
-    if (len < 2 || isdigit(filepath[len - 2]) || filepath[len - 1] != '.') {
+    if (len < 2 || gv_isdigit(filepath[len - 2]) || filepath[len - 1] != '.') {
 	return false;
     }
 #elif defined(__MINGW32__) || defined(__CYGWIN__)
-    if (len < 2 || isdigit((int)filepath[len - 2]) || filepath[len - 1] != '-') {
+    if (len < 2 || gv_isdigit(filepath[len - 2]) || filepath[len - 1] != '-') {
 	return false;
     }
 #elif defined(_WIN32)
-    if (len < 1 || isdigit((int)filepath[len - 1])) {
+    if (len < 1 || gv_isdigit(filepath[len - 1])) {
 	return false;
     }
 #elif ((defined(__hpux__) || defined(__hpux)) && !(defined(__ia64)))
     static const char SL[] = ".sl.";
-    if (len < strlen(SL)
-        || strncmp(filepath + len - strlen(SL), SL, strlen(SL)) != 0) {
+    if (len < strlen(SL) || !startswith(filepath + len - strlen(SL), SL)) {
 	return false;
     }
 #else
     static const char SO[] = ".so.";
-    if (len < strlen(SO)
-        || strncmp(filepath + len - strlen(SO), SO, strlen(SO)) != 0) {
+    if (len < strlen(SO) || !startswith(filepath + len - strlen(SO), SO)) {
 	return false;
     }
 #endif
@@ -472,7 +474,7 @@ static void config_rescan(GVC_t *gvc, char *config_path)
 {
     FILE *f = NULL;
     glob_t globbuf;
-    char *config_glob, *libdir;
+    char *libdir;
     int rc;
     gvplugin_library_t *library;
 #if defined(DARWIN_DYLIB)
@@ -503,17 +505,15 @@ static void config_rescan(GVC_t *gvc, char *config_path)
 
     libdir = gvconfig_libdir(gvc);
 
-    config_glob = gmalloc(strlen(libdir) + 1 + strlen(plugin_glob) + 1);
-    strcpy(config_glob, libdir);
-	strcat(config_glob, DIRSEP);
-    strcat(config_glob, plugin_glob);
+    agxbuf config_glob = {0};
+    agxbprint(&config_glob, "%s%s%s", libdir, DIRSEP, plugin_glob);
 
     /* load all libraries even if can't save config */
 
 #if defined(_WIN32)
-    rc = glob(gvc, config_glob, GLOB_NOSORT, NULL, &globbuf);
+    rc = glob(gvc, agxbuse(&config_glob), GLOB_NOSORT, NULL, &globbuf);
 #else
-    rc = glob(config_glob, 0, NULL, &globbuf);
+    rc = glob(agxbuse(&config_glob), 0, NULL, &globbuf);
 #endif
     if (rc == 0) {
 	for (size_t i = 0; i < globbuf.gl_pathc; i++) {
@@ -539,7 +539,7 @@ static void config_rescan(GVC_t *gvc, char *config_path)
 	}
     }
     globfree(&globbuf);
-    free(config_glob);
+    agxbfree(&config_glob);
     if (f)
 	fclose(f);
 }
@@ -552,7 +552,7 @@ void gvconfig(GVC_t * gvc, bool rescan)
 {
 #ifdef ENABLE_LTDL
     int rc;
-    struct stat config_st, libdir_st;
+    struct stat config_st;
     FILE *f = NULL;
     char *config_text = NULL;
     char *libdir;
@@ -568,24 +568,23 @@ void gvconfig(GVC_t * gvc, bool rescan)
     if (gvc->common.demand_loading) {
         /* see if there are any new plugins */
         libdir = gvconfig_libdir(gvc);
-        rc = stat(libdir, &libdir_st);
-        if (rc == -1) {
-	    gvtextlayout_select(gvc);   /* choose best available textlayout plugin immediately */
+        if (access(libdir, F_OK) < 0) {
     	    /* if we fail to stat it then it probably doesn't exist so just fail silently */
-	    return;
+	    goto done;
         }
     
         if (! gvc->config_path) {
-            gvc->config_path = gmalloc(strlen(libdir) + 1 + strlen(config_file_name) + 1);
-            strcpy(gvc->config_path, libdir);
-            strcat(gvc->config_path, DIRSEP);
-            strcat(gvc->config_path, config_file_name);
+            agxbuf xb = {0};
+            agxbprint(&xb, "%s%s%s", libdir, DIRSEP, config_file_name);
+            gvc->config_path = agxbdisown(&xb);
         }
     	
         if (rescan) {
     	    config_rescan(gvc, gvc->config_path);
     	    gvc->config_found = true;
 	    gvtextlayout_select(gvc);   /* choose best available textlayout plugin immediately */
+	    assert(gvc->textfont_dt != NULL &&
+	           "config rescan performed without any prior first scan");
     	    return;
         }
     
@@ -593,9 +592,8 @@ void gvconfig(GVC_t * gvc, bool rescan)
     
         rc = stat(gvc->config_path, &config_st);
         if (rc == -1) {
-	    gvtextlayout_select(gvc);   /* choose best available textlayout plugin immediately */
     	    /* silently return without setting gvc->config_found = TRUE */
-    	    return;
+    	    goto done;
         }
         else {
     	    f = fopen(gvc->config_path,"r");
@@ -607,7 +605,7 @@ void gvconfig(GVC_t * gvc, bool rescan)
     	        agerr(AGERR, "%s is zero sized.\n", gvc->config_path);
     	    }
     	    else {
-    	        config_text = gmalloc((size_t)config_st.st_size + 1);
+    	        config_text = gv_alloc((size_t)config_st.st_size + 1);
     	        size_t sz = fread(config_text, 1, (size_t)config_st.st_size, f);
     	        if (sz == 0) {
     	            agerr(AGERR, "%s read error.\n", gvc->config_path);
@@ -624,6 +622,7 @@ void gvconfig(GVC_t * gvc, bool rescan)
 	    }
         }
     }
+done:
 #endif
     gvtextlayout_select(gvc);   /* choose best available textlayout plugin immediately */
     textfont_dict_open(gvc);    /* initialize font dict */
@@ -658,11 +657,12 @@ glob (GVC_t* gvc, char* pattern, int flags, int (*errfunc)(const char *, int), g
     do {
       if (cnt >= arrsize-1) {
         arrsize += 512;
-        str = realloc (str, arrsize*sizeof(char*));
-        if (!str) return GLOB_NOSPACE;
+        char **new_str = realloc(str, arrsize * sizeof(char*));
+        if (!new_str) goto oom;
+        str = new_str;
       }
       str[cnt] = malloc (strlen(libdir)+1+strlen(wfd.cFileName)+1);
-      if (!str[cnt]) return GLOB_NOSPACE;
+      if (!str[cnt]) goto oom;
       strcpy(str[cnt],libdir);
       strcat(str[cnt],DIRSEP);
       strcat(str[cnt],wfd.cFileName);
@@ -674,6 +674,12 @@ glob (GVC_t* gvc, char* pattern, int flags, int (*errfunc)(const char *, int), g
     pglob->gl_pathv = realloc(str, (cnt+1)*sizeof(char*));
     
     return 0;
+
+oom:
+    for (int i = 0; i < cnt; ++i)
+      free(str[i]);
+    free(str);
+    return GLOB_NOSPACE;
 }
 
 static void
