@@ -24,16 +24,6 @@ extern "C" {
 
     /* some possible values for 'num_pivots_stress' */
 #define num_pivots_stress 40
-#define num_pivots_smart_ini   0
-#define num_pivots_no_ini   50
-
-    /* relevant when using sparse distance matrix
-     * when optimizing within subspace it can be set to 0
-     * otherwise, recommended value is above zero (usually around 3-6)
-     * some possible values for 'neighborhood_radius'
-     */
-#define neighborhood_radius_unrestricted  4
-#define neighborhood_radius_subspace 0
 
 #define opt_smart_init 0x4
 #define opt_exp_flag   0x3
@@ -51,7 +41,7 @@ extern "C" {
 	);
 
 extern float *compute_apsp_packed(vtx_data * graph, int n);
-extern float *compute_apsp_artifical_weights_packed(vtx_data * graph, int n);
+extern float *compute_apsp_artificial_weights_packed(vtx_data *graph, int n);
 extern float* circuitModel(vtx_data * graph, int nG);
 extern float* mdsModel (vtx_data * graph, int nG);
 extern int initLayout(int n, int dim, double **coords, node_t **nodes);
