@@ -1,6 +1,3 @@
-/* $Id$ $Revision$ */
-/* vim:set shiftwidth=4 ts=8: */
-
 /*************************************************************************
  * Copyright (c) 2011 AT&T Intellectual Property 
  * All rights reserved. This program and the accompanying materials
@@ -8,7 +5,7 @@
  * which accompanies this distribution, and is available at
  * http://www.eclipse.org/legal/epl-v10.html
  *
- * Contributors: See CVS logs. Details at http://www.graphviz.org/
+ * Contributors: Details at https://graphviz.org
  *************************************************************************/
 
 /* 
@@ -19,15 +16,11 @@
  */
 
 #include <stdlib.h>
-#include "vispath.h"
-#include "pathutil.h"
+#include <pathplan/vispath.h>
+#include <pathplan/pathutil.h>
+#include <stdbool.h>
 
-#ifdef DMALLOC
-#include "dmalloc.h"
-#endif
-
-int in_poly(Ppoly_t poly, Ppoint_t q)
-{
+bool in_poly(Ppoly_t poly, Ppoint_t q) {
     int i, i1;			/* point index; i1 = i-1 mod n */
     int n;
     Ppoint_t *P;
@@ -36,7 +29,7 @@ int in_poly(Ppoly_t poly, Ppoint_t q)
     n = poly.pn;
     for (i = 0; i < n; i++) {
 	i1 = (i + n - 1) % n;
-	if (wind(P[i1],P[i],q) == 1) return FALSE;
+	if (wind(P[i1], P[i], q) == 1) return false;
     }
-    return TRUE;
+    return true;
 }
