@@ -10,37 +10,32 @@
 
 #pragma once
 
-#define _BLD_pathplan 1
-
 #include "pathplan.h"
+#include <stdbool.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-#ifndef NOT
-#define NOT(x)	(!(x))
+#ifdef GVDLL
+#ifdef PATHPLAN_EXPORTS
+#define PATHUTIL_API __declspec(dllexport)
+#else
+#define PATHUTIL_API __declspec(dllimport)
 #endif
-#ifndef FALSE
-#define FALSE	0
-#define TRUE	(NOT(FALSE))
 #endif
 
-/*visual studio*/
-#ifdef _WIN32
-#ifndef PATHPLAN_EXPORTS
-#define extern __declspec(dllimport)
+#ifndef PATHUTIL_API
+#define PATHUTIL_API /* nothing */
 #endif
-#endif
-/*end visual studio*/
 	typedef double COORD;
-    extern COORD area2(Ppoint_t, Ppoint_t, Ppoint_t);
-    extern int wind(Ppoint_t a, Ppoint_t b, Ppoint_t c);
-    extern COORD dist2(Ppoint_t, Ppoint_t);
+    PATHUTIL_API COORD area2(Ppoint_t, Ppoint_t, Ppoint_t);
+    PATHUTIL_API int wind(Ppoint_t a, Ppoint_t b, Ppoint_t c);
+    PATHUTIL_API COORD dist2(Ppoint_t, Ppoint_t);
 
-    int in_poly(Ppoly_t argpoly, Ppoint_t q);
+    PATHUTIL_API bool in_poly(Ppoly_t argpoly, Ppoint_t q);
 
-#undef extern
+#undef PATHUTIL_API
 #ifdef __cplusplus
 }
 #endif
