@@ -11,8 +11,9 @@
 #pragma once
 
 #include <sfdpgen/spring_electrical.h>
+#include <stdbool.h>
 
-enum {SM_SCHEME_NORMAL, SM_SCHEME_NORMAL_ELABEL, SM_SCHEME_UNIFORM_STRESS, SM_SCHEME_MAXENT, SM_SCHEME_STRESS_APPROX, SM_SCHEME_STRESS};
+enum {SM_SCHEME_NORMAL, SM_SCHEME_NORMAL_ELABEL, SM_SCHEME_STRESS};
 
 struct StressMajorizationSmoother_struct {
   SparseMatrix D;/* distance matrix. The diagonal is removed hence the ia, ja structure is different from Lw and Lwd!! */
@@ -36,7 +37,7 @@ void StressMajorizationSmoother_delete(StressMajorizationSmoother sm);
 enum {IDEAL_GRAPH_DIST, IDEAL_AVG_DIST, IDEAL_POWER_DIST};
 StressMajorizationSmoother StressMajorizationSmoother2_new(SparseMatrix A, int dim, double lambda, double *x, int ideal_dist_scheme);
 
-double StressMajorizationSmoother_smooth(StressMajorizationSmoother sm, int dim, double *x, int maxit, double tol);
+double StressMajorizationSmoother_smooth(StressMajorizationSmoother sm, int dim, double *x, int maxit);
 /*-------------------- triangle/neirhborhood graph based smoother ------------------- */
 typedef  StressMajorizationSmoother TriangleSmoother;
 
@@ -44,7 +45,8 @@ typedef  StressMajorizationSmoother TriangleSmoother;
 
 void TriangleSmoother_delete(TriangleSmoother sm);
 
-TriangleSmoother TriangleSmoother_new(SparseMatrix A, int dim, double lambda, double *x, int use_triangularization);
+TriangleSmoother TriangleSmoother_new(SparseMatrix A, int dim, double *x,
+                                      bool use_triangularization);
 
 void TriangleSmoother_smooth(TriangleSmoother sm, int dim, double *x);
 
@@ -63,10 +65,10 @@ SpringSmoother SpringSmoother_new(SparseMatrix A, int dim, spring_electrical_con
 
 void SpringSmoother_delete(SpringSmoother sm);
 
-void SpringSmoother_smooth(SpringSmoother sm, SparseMatrix A, double *node_weights, int dim, double *x);
+void SpringSmoother_smooth(SpringSmoother sm, SparseMatrix A, int dim, double *x);
 /*------------------------------------------------------------------*/
 
-void post_process_smoothing(int dim, SparseMatrix A, spring_electrical_control ctrl, double *node_weights, double *x, int *flag);
+void post_process_smoothing(int dim, SparseMatrix A, spring_electrical_control ctrl, double *x);
 
 /*-------------------- sparse stress majorizationp ------------------- */
 typedef  StressMajorizationSmoother SparseStressMajorizationSmoother;
@@ -75,12 +77,9 @@ typedef  StressMajorizationSmoother SparseStressMajorizationSmoother;
 
 void SparseStressMajorizationSmoother_delete(SparseStressMajorizationSmoother sm);
 
-enum {WEIGHTING_SCHEME_NONE, WEIGHTING_SCHEME_INV_DIST, WEIGHTING_SCHEME_SQR_DIST};
-SparseStressMajorizationSmoother SparseStressMajorizationSmoother_new(SparseMatrix A, int dim, double lambda, double *x, 
-								      int weighting_scheme, int scale_initial_coord);
+SparseStressMajorizationSmoother
+SparseStressMajorizationSmoother_new(SparseMatrix A, int dim, double *x);
 
-double SparseStressMajorizationSmoother_smooth(SparseStressMajorizationSmoother sm, int dim, double *x, int maxit_sm, double tol);
-
-double get_stress(int m, int dim, int *iw, int *jw, double *w, double *d, double *x, double scaling, void *data, int weighted);
+double SparseStressMajorizationSmoother_smooth(SparseStressMajorizationSmoother sm, int dim, double *x, int maxit_sm);
 
 /*--------------------------------------------------------------*/
