@@ -10,6 +10,12 @@
 
 #pragma once
 
+#include <stdbool.h>
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef struct Multilevel_Modularity_Clustering_struct *Multilevel_Modularity_Clustering;
 
 struct Multilevel_Modularity_Clustering_struct {
@@ -20,16 +26,15 @@ struct Multilevel_Modularity_Clustering_struct {
   SparseMatrix R; 
   Multilevel_Modularity_Clustering next;
   Multilevel_Modularity_Clustering prev;
-  int delete_top_level_A;
+  bool delete_top_level_A;
   int *matching; /* dimension n. matching[i] is the clustering assignment of node i */
-  real modularity;
-  real deg_total; /* total edge weights, including self-edges */
-  real *deg;/* dimension n. deg[i] equal to the sum of edge weights connected to vertex i. I.e., sum of  row i */
-  int agglomerate_regardless;/* whether to agglomerate nodes even if this causes modularity reduction. This is used if we want to force
-				agglomeration so as to get less clusters
-			      */
-
-
+  double modularity;
+  double deg_total; /* total edge weights, including self-edges */
+  double *deg;/* dimension n. deg[i] equal to the sum of edge weights connected to vertex i. I.e., sum of  row i */
+  bool agglomerate_regardless; ///< whether to agglomerate nodes even if this
+                               ///< causes modularity reduction. This is used if
+                               ///< we want to force agglomeration so as to get
+                               ///< fewer clusters
 };
 
 enum {CLUSTERING_MODULARITY = 0, CLUSTERING_MQ};
@@ -41,11 +46,14 @@ enum {CLUSTERING_MODULARITY = 0, CLUSTERING_MQ};
    maxcluster: used to specify the maximum number of cluster desired, e.g., maxcluster=10 means that a maximum of 10 clusters
    .   is desired. this may not always be realized, and modularity may be low when this is specified. Default: maxcluster = 0 (no limit)
 
-   use_value: whether to use the entry value, or treat edge weights as 1.
    nclusters: on output the number of clusters
    assignment: dimension n. Node i is assigned to cluster "assignment[i]". 0 <= assignment < nclusters.
    .   If *assignment = NULL on entry, it will be allocated. Otherwise used.
    modularity: achieve modularity
 */
-void modularity_clustering(SparseMatrix A, int inplace, int maxcluster, int use_value,
-			   int *nclusters, int **assignment, real *modularity, int *flag);
+void modularity_clustering(SparseMatrix A, bool inplace, int maxcluster,
+			   int *nclusters, int **assignment, double *modularity);
+
+#ifdef __cplusplus
+}
+#endif
