@@ -10,12 +10,12 @@
 
 #include <cgraph/agxbuf.h>
 #include <cgraph/alloc.h>
+#include <cgraph/gv_ctype.h>
 #include <cgraph/prisize_t.h>
 #include <cgraph/unreachable.h>
 #include <xdot/xdot.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 
 /* the parse functions should return NULL on error */
 static char *parseReal(char *s, double *fp)
@@ -128,14 +128,26 @@ static char *parseString(char *s, char **sp)
 	return 0;
     }
 
-    char *c = gv_strndup(s, (size_t)i);
-    if (strlen(c) != (size_t)i) {
-	free (c);
-	return 0;
+    // The leading number we just read indicates the count of originating
+    // characters in the upcoming string. But the string may contain \-escaped
+    // characters. So the count alone does not tell us how many bytes we need to
+    // now read.
+    agxbuf c = {0};
+    int j = 0;
+    for (int accounted = 0; accounted < i; ++j) {
+	if (s[j] == '\0') {
+	    agxbfree(&c);
+	    return 0;
+	}
+	agxbputc(&c, s[j]);
+	// only count this character if it was not an escape prefix
+	if (s[j] != '\\' || (j > 0 && s[j - 1] == '\\')) {
+	    ++accounted;
+	}
     }
 
-    *sp = c;
-    return s + i;
+    *sp = agxbdisown(&c);
+    return s + j;
 }
 
 static char *parseAlign(char *s, xdot_align * ap)
@@ -160,7 +172,7 @@ static char *parseOp(xdot_op * op, char *s, drawfunc_t ops[], int* error)
     xdot_color clr;
 
     *error = 0;
-    while (isspace((int)*s))
+    while (gv_isspace(*s))
 	s++;
     switch (*s++) {
     case 'E':
@@ -353,9 +365,8 @@ xdot *parseXDotFOn(char *s, drawfunc_t fns[], size_t sz, xdot *x) {
 	ops = gv_calloc(XDBSIZE, sz);
     }
     else {
-	ops = (char*)(x->ops);
 	bufsz = initcnt + XDBSIZE;
-	ops = gv_recalloc(ops, initcnt, bufsz, sz);
+	ops = gv_recalloc(x->ops, initcnt, bufsz, sz);
     }
 
     while ((s = parseOp(&op, s, fns, &error))) {
@@ -393,57 +404,39 @@ xdot *parseXDot(char *s)
 
 typedef int (*pf)(void*, char*, ...);
 
-/* trim:
- * Trailing zeros are removed and decimal point, if possible.
- */
-static void trim (char* buf)
-{
-    char* dotp;
-    char* p;
-
-    if ((dotp = strchr (buf,'.'))) {
-        p = dotp+1;
-        while (*p) p++;  // find end of string
-        p--;
-        while (*p == '0') *p-- = '\0';
-        if (*p == '.')        // If all decimals were zeros, remove ".".
-            *p = '\0';
-        else
-            p++;
-    }
-}
-
 static void printRect(xdot_rect * r, pf print, void *info)
 {
-    char buf[128];
+    agxbuf buf = {0};
 
-    snprintf(buf, sizeof(buf), " %.02f", r->x);
-    trim(buf);
-    print(info, "%s", buf);
-    snprintf(buf, sizeof(buf), " %.02f", r->y);
-    trim(buf);
-    print(info, "%s", buf);
-    snprintf(buf, sizeof(buf), " %.02f", r->w);
-    trim(buf);
-    print(info, "%s", buf);
-    snprintf(buf, sizeof(buf), " %.02f", r->h);
-    trim(buf);
-    print(info, "%s", buf);
+    agxbprint(&buf, " %.02f", r->x);
+    agxbuf_trim_zeros(&buf);
+    print(info, "%s", agxbuse(&buf));
+    agxbprint(&buf, " %.02f", r->y);
+    agxbuf_trim_zeros(&buf);
+    print(info, "%s", agxbuse(&buf));
+    agxbprint(&buf, " %.02f", r->w);
+    agxbuf_trim_zeros(&buf);
+    print(info, "%s", agxbuse(&buf));
+    agxbprint(&buf, " %.02f", r->h);
+    agxbuf_trim_zeros(&buf);
+    print(info, "%s", agxbuse(&buf));
+    agxbfree(&buf);
 }
 
 static void printPolyline(xdot_polyline * p, pf print, void *info)
 {
-    char buf[512];
+    agxbuf buf = {0};
 
     print(info, " %" PRISIZE_T, p->cnt);
     for (size_t i = 0; i < p->cnt; i++) {
-	snprintf(buf, sizeof(buf), " %.02f", p->pts[i].x);
-	trim(buf);
-	print(info, "%s", buf);
-	snprintf(buf, sizeof(buf), " %.02f", p->pts[i].y);
-	trim(buf);
-	print(info, "%s", buf);
+	agxbprint(&buf, " %.02f", p->pts[i].x);
+	agxbuf_trim_zeros(&buf);
+	print(info, "%s", agxbuse(&buf));
+	agxbprint(&buf, " %.02f", p->pts[i].y);
+	agxbuf_trim_zeros(&buf);
+	print(info, "%s", agxbuse(&buf));
     }
+    agxbfree(&buf);
 }
 
 static void printString(char *p, pf print, void *info)
@@ -452,14 +445,15 @@ static void printString(char *p, pf print, void *info)
 }
 
 static void printFloat(double f, pf print, void *info, int space) {
-    char buf[128];
+    agxbuf buf = {0};
 
     if (space)
-	snprintf(buf, sizeof(buf), " %.02f", f);
+	agxbprint(&buf, " %.02f", f);
     else
-	snprintf(buf, sizeof(buf), "%.02f", f);
-    trim (buf);
-    print(info, "%s", buf);
+	agxbprint(&buf, "%.02f", f);
+    agxbuf_trim_zeros(&buf);
+    print(info, "%s", agxbuse(&buf));
+    agxbfree(&buf);
 }
 
 static void printAlign(xdot_align a, pf print, void *info)
@@ -521,10 +515,7 @@ typedef void (*print_op)(xdot_op * op, pf print, void *info, int more);
 
 static void printXDot_Op(xdot_op * op, pf print, void *info, int more)
 {
-    agxbuf xb;
-    char buf[BUFSIZ];
-
-    agxbinit (&xb, BUFSIZ, buf);
+    agxbuf xb = {0};
     switch (op->kind) {
     case xd_filled_ellipse:
 	print(info, "E");
@@ -631,10 +622,7 @@ static void jsonString(char *p, pf print, void *info)
 
 static void jsonXDot_Op(xdot_op * op, pf print, void *info, int more)
 {
-    agxbuf xb;
-    char buf[BUFSIZ];
-
-    agxbinit (&xb, BUFSIZ, buf);
+    agxbuf xb = {0};
     switch (op->kind) {
     case xd_filled_ellipse:
 	print(info, "{\"E\" : ");
@@ -731,9 +719,7 @@ static void _printXDot(xdot * x, pf print, void *info, print_op ofn)
 
 char *sprintXDot(xdot * x)
 {
-    char buf[BUFSIZ];
-    agxbuf xb;
-    agxbinit(&xb, BUFSIZ, buf);
+    agxbuf xb = {0};
     _printXDot(x, (pf)agxbprint, &xb, printXDot_Op);
     return agxbdisown(&xb);
 }
@@ -971,7 +957,7 @@ parseXDotColor (char* cp, xdot_color* clr)
 	return cp;
 	break;
     default :
-	if (isalnum(c)) {
+	if (gv_isalnum(c)) {
 	    clr->type = xd_none; 
 	    clr->u.clr = cp;
 	    return cp;
@@ -998,3 +984,11 @@ void freeXDotColor (xdot_color* cp)
 	free (cp->u.ring.stops);
     }
 }
+
+/**
+ * @dir lib/xdot
+ * @brief parsing and deparsing of xdot operations, API xdot.h
+ *
+ * [man 3 xdot](https://graphviz.org/pdf/xdot.3.pdf)
+ *
+ */
