@@ -14,12 +14,10 @@
 #include <io.h>
 #endif
 
-#include <limits.h>
 #include <stdarg.h>
 #include <stdbool.h>
 #include <stdlib.h>
 #include <string.h>
-#include <ctype.h>
 
 #include <common/macros.h>
 #include <common/const.h>
@@ -28,7 +26,9 @@
 #include <gvc/gvplugin_device.h>
 #include <cgraph/alloc.h>
 #include <cgraph/agxbuf.h>
+#include <cgraph/gv_ctype.h>
 #include <cgraph/prisize_t.h>
+#include <cgraph/streq.h>
 #include <cgraph/unreachable.h>
 #include <common/utils.h>
 #include <gvc/gvc.h>
@@ -75,7 +75,6 @@ typedef struct {
     attrsym_t *e_l_draw;
     attrsym_t *hl_draw;
     attrsym_t *tl_draw;
-    char buf[NUMXBUFS][BUFSIZ];
     unsigned short version;
     char* version_s;
 } xdot_state_t;
@@ -113,13 +112,10 @@ static void xdot_num(agxbuf *xb, double v)
   xdot_fmt_num(xb, v);
 }
 
-static void xdot_points(GVJ_t *job, char c, pointf * A, int n)
-{
+static void xdot_points(GVJ_t *job, char c, pointf *A, size_t n) {
     emit_state_t emit_state = job->obj->emit_state;
-    int i;
-
-    agxbprint(xbufs[emit_state], "%c %d ", c, n);
-    for (i = 0; i < n; i++)
+    agxbprint(xbufs[emit_state], "%c %" PRISIZE_T " ", c, n);
+    for (size_t i = 0; i < n; i++)
         xdot_point(xbufs[emit_state], A[i]);
 }
 
@@ -281,7 +277,7 @@ static void xdot_end_cluster(GVJ_t * job)
 static unsigned short versionStr2Version(const char *str) {
   unsigned short us = 0;
   for (size_t i = 0; str[i] != '\0'; ++i) {
-    if (!isdigit((int)str[i])) {
+    if (!gv_isdigit(str[i])) {
       continue;
     }
     unsigned short digit = (unsigned short)(str[i] - '0');
@@ -377,7 +373,7 @@ xdot_begin_graph (graph_t *g, int s_arrows, int e_arrows, format_type id)
 	xd->tl_draw = NULL;
 
     for (i = 0; i < NUMXBUFS; i++)
-	agxbinit(xbuf+i, BUFSIZ, xd->buf[i]);
+	xbuf[i] = (agxbuf){0};
 }
 
 static void dot_begin_graph(GVJ_t *job)
@@ -526,10 +522,8 @@ static void xdot_color_stop (agxbuf* xb, float v, gvcolor_t* clr)
   xdot_str_xbuf(xb, "", color2str (clr->u.rgba));
 }
 
-static void xdot_gradient_fillcolor (GVJ_t* job, int filled, pointf* A, int n)
+static void xdot_gradient_fillcolor(GVJ_t *job, int filled, pointf *A, size_t n)
 {
-    char buf0[BUFSIZ];
-    agxbuf xb;
     obj_state_t* obj = job->obj;
     double angle = obj->gradient_angle * M_PI / 180;
     pointf G[2],c1,c2;
@@ -539,7 +533,7 @@ static void xdot_gradient_fillcolor (GVJ_t* job, int filled, pointf* A, int n)
 	return;
     }
 
-    agxbinit(&xb, BUFSIZ, buf0);
+    agxbuf xb = {0};
     if (filled == GRADIENT) {
 	get_gradient_points(A, G, n, angle, 2);
 	agxbputc (&xb, '[');
@@ -607,28 +601,27 @@ static void xdot_ellipse(GVJ_t * job, pointf * A, int filled)
     xdot_fmt_num(xbufs[emit_state], A[1].y - A[0].y);
 }
 
-static void xdot_bezier(GVJ_t *job, pointf *A, int n, int filled) {
+static void xdot_bezier(GVJ_t *job, pointf *A, size_t n, int filled) {
     xdot_style (job);
     xdot_pencolor (job);
     if (filled) {
 	if ((filled == GRADIENT) || (filled == RGRADIENT)) {
-	   xdot_gradient_fillcolor (job, filled, A, n);
+	   xdot_gradient_fillcolor(job, filled, A, n);
 	}
         else
 	    xdot_fillcolor (job);
-        xdot_points(job, 'b', A, n);   /* NB - 'B' & 'b' are reversed in comparison to the other items */
+        xdot_points(job, 'b', A, n); // NB - 'B' & 'b' are reversed in comparison to the other items
     }
     else
         xdot_points(job, 'B', A, n);
 }
 
-static void xdot_polygon(GVJ_t * job, pointf * A, int n, int filled)
-{
+static void xdot_polygon(GVJ_t *job, pointf *A, size_t n, int filled) {
     xdot_style (job);
     xdot_pencolor (job);
     if (filled) {
 	if ((filled == GRADIENT) || (filled == RGRADIENT)) {
-	   xdot_gradient_fillcolor (job, filled, A, n);
+	   xdot_gradient_fillcolor(job, filled, A, n);
 	}
         else
 	    xdot_fillcolor (job);
@@ -638,8 +631,7 @@ static void xdot_polygon(GVJ_t * job, pointf * A, int n, int filled)
         xdot_points(job, 'p', A, n);
 }
 
-static void xdot_polyline(GVJ_t * job, pointf * A, int n)
-{
+static void xdot_polyline(GVJ_t *job, pointf *A, size_t n) {
     xdot_style (job);
     xdot_pencolor (job);
     xdot_points(job, 'L', A, n);
