@@ -19,9 +19,10 @@
  */
 
 #include "config.h"
-
 #include <math.h>
 #include <stdarg.h>
+#include <stdbool.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -44,7 +45,7 @@
   #define EDGEALIGN 0
 #endif
 
-typedef enum { FORMAT_SVG, FORMAT_SVGZ, } format_type;
+typedef enum { FORMAT_SVG, FORMAT_SVGZ, FORMAT_SVG_INLINE } format_type;
 
 /* SVG dash array */
 static const char sdasharray[] = "5,2";
@@ -55,16 +56,18 @@ static const char transparent[] = "transparent";
 static const char none[] = "none";
 static const char black[] = "black";
 
-static void svg_bzptarray(GVJ_t * job, pointf * A, int n)
-{
-    int i;
+static bool emit_standalone_headers(const GVJ_t *job) {
+  return job->render.id != FORMAT_SVG_INLINE;
+}
+
+static void svg_bzptarray(GVJ_t *job, pointf *A, size_t n) {
     char c;
 
     c = 'M';			/* first point */
 #if EDGEALIGN
     if (A[0].x <= A[n-1].x) {
 #endif
-	for (i = 0; i < n; i++) {
+	for (size_t i = 0; i < n; i++) {
 	    gvwrite(job, &c, 1);
             gvprintdouble(job, A[i].x);
             gvputc(job, ',');
@@ -76,7 +79,7 @@ static void svg_bzptarray(GVJ_t * job, pointf * A, int n)
 	}
 #if EDGEALIGN
     } else {
-	for (i = n-1; i >= 0; i--) {
+	for (size_t i = n - 1; i != SIZE_MAX; i--) {
 	    gvwrite(job, &c, 1);
             gvprintdouble(job, A[i].x);
             gvputc(job, ',');
@@ -217,17 +220,18 @@ static void svg_comment(GVJ_t * job, char *str)
 static void svg_begin_job(GVJ_t * job)
 {
     char *s;
-    gvputs(job,
-	   "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n");
-    if ((s = agget(job->gvc->g, "stylesheet")) && s[0]) {
-	gvputs(job, "<?xml-stylesheet href=\"");
-	gvputs(job, s);
-	gvputs(job, "\" type=\"text/css\"?>\n");
+    if (emit_standalone_headers(job)) {
+	    gvputs(job,
+		   "<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"no\"?>\n");
+	    if ((s = agget(job->gvc->g, "stylesheet")) && s[0]) {
+		gvputs(job, "<?xml-stylesheet href=\"");
+		gvputs(job, s);
+		gvputs(job, "\" type=\"text/css\"?>\n");
+	    }
+	    gvputs(job, "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"\n"
+                " \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n");
     }
-    gvputs(job, "<!DOCTYPE svg PUBLIC \"-//W3C//DTD SVG 1.1//EN\"\n"
-                " \"http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd\">\n"
-
-                "<!-- Generated by ");
+    gvputs(job,"<!-- Generated by ");
     gvputs_xml(job, job->common->info[0]);
     gvputs(job, " version ");
     gvputs_xml(job, job->common->info[1]);
@@ -256,11 +260,18 @@ static void svg_begin_graph(GVJ_t * job)
 	job->canvasBox.LL.y,
 	job->canvasBox.UR.x,
 	job->canvasBox.UR.y);
-    /* namespace of svg */
-    gvputs(job, " xmlns=\"http://www.w3.org/2000/svg\""
-    /* namespace of xlink */
-                " xmlns:xlink=\"http://www.w3.org/1999/xlink\""
-                ">\n");
+    // https://svgwg.org/svg2-draft/struct.html#Namespace says:
+    // > There's no need to have an ‘xmlns’ attribute declaring that the
+    // > element is in the SVG namespace when using the HTML parser. The HTML
+    // > parser will automatically create the SVG elements in the proper
+    // > namespace.
+    if (emit_standalone_headers(job)) {
+	/* namespace of svg */
+	gvputs(job, " xmlns=\"http://www.w3.org/2000/svg\""
+	/* namespace of xlink */
+	    " xmlns:xlink=\"http://www.w3.org/1999/xlink\"");
+    }
+    gvputs(job, ">\n");
 }
 
 static void svg_end_graph(GVJ_t * job)
@@ -475,7 +486,7 @@ static void svg_textspan(GVJ_t * job, pointf p, textspan_t * span)
 	    gvprintf(job, " font-style=\"%s\"", style);
     } else
 	gvprintf(job, " font-family=\"%s\"", span->font->name);
-    if (span->font && (flags = span->font->flags)) {
+    if ((flags = span->font->flags)) {
 	if ((flags & HTML_BF) && !weight)
 	    gvputs(job, " font-weight=\"bold\"");
 	if ((flags & HTML_IF) && !style)
@@ -554,8 +565,7 @@ static void svg_print_stop(GVJ_t * job, double offset, gvcolor_t color)
 /* svg_gradstyle
  * Outputs the SVG statements that define the gradient pattern
  */
-static int svg_gradstyle(GVJ_t * job, pointf * A, int n)
-{
+static int svg_gradstyle(GVJ_t *job, pointf *A, size_t n) {
     pointf G[2];
     static int gradId;
     int id = gradId++;
@@ -563,7 +573,7 @@ static int svg_gradstyle(GVJ_t * job, pointf * A, int n)
     obj_state_t *obj = job->obj;
     double angle = obj->gradient_angle * M_PI / 180; //angle of gradient line
     G[0].x = G[0].y = G[1].x = G[1].y = 0.;
-    get_gradient_points(A, G, n, angle, 0);	//get points on gradient line
+    get_gradient_points(A, G, n, angle, 0); // get points on gradient line
 
     gvputs(job, "<defs>\n<linearGradient id=\"");
     if (obj->id != NULL) {
@@ -645,8 +655,7 @@ static void svg_ellipse(GVJ_t * job, pointf * A, int filled)
     gvputs(job, "\"/>\n");
 }
 
-static void
-svg_bezier(GVJ_t *job, pointf *A, int n, int filled) {
+static void svg_bezier(GVJ_t *job, pointf *A, size_t n, int filled) {
     int gid = 0;
     obj_state_t *obj = job->obj;
   
@@ -667,9 +676,8 @@ svg_bezier(GVJ_t *job, pointf *A, int n, int filled) {
     gvputs(job, "\"/>\n");
 }
 
-static void svg_polygon(GVJ_t * job, pointf * A, int n, int filled)
-{
-    int i, gid = 0;
+static void svg_polygon(GVJ_t *job, pointf *A, size_t n, int filled) {
+    int gid = 0;
     if (filled == GRADIENT) {
 	gid = svg_gradstyle(job, A, n);
     } else if (filled == RGRADIENT) {
@@ -678,7 +686,7 @@ static void svg_polygon(GVJ_t * job, pointf * A, int n, int filled)
     gvputs(job, "<polygon");
     svg_grstyle(job, filled, gid);
     gvputs(job, " points=\"");
-    for (i = 0; i < n; i++) {
+    for (size_t i = 0; i < n; i++) {
         gvprintdouble(job, A[i].x);
         gvputc(job, ',');
         gvprintdouble(job, -A[i].y);
@@ -691,18 +699,15 @@ static void svg_polygon(GVJ_t * job, pointf * A, int n, int filled)
     gvputs(job, "\"/>\n");
 }
 
-static void svg_polyline(GVJ_t * job, pointf * A, int n)
-{
-    int i;
-
+static void svg_polyline(GVJ_t *job, pointf *A, size_t n) {
     gvputs(job, "<polyline");
     svg_grstyle(job, 0, 0);
     gvputs(job, " points=\"");
-    for (i = 0; i < n; i++) {
+    for (size_t i = 0; i < n; i++) {
         gvprintdouble(job, A[i].x);
         gvputc(job, ',');
         gvprintdouble(job, -A[i].y);
-        if (i != n - 1) {
+        if (i + 1 != n) {
             gvputc(job, ' ');
         }
     }
@@ -811,6 +816,7 @@ gvdevice_features_t device_features_svgz = {
 
 gvplugin_installed_t gvrender_svg_types[] = {
     {FORMAT_SVG, "svg", 1, &svg_engine, &render_features_svg},
+    {FORMAT_SVG_INLINE, "svg_inline", 1, &svg_engine, &render_features_svg},
     {0, NULL, 0, NULL, NULL}
 };
 
@@ -819,5 +825,6 @@ gvplugin_installed_t gvdevice_svg_types[] = {
 #ifdef HAVE_LIBZ
     {FORMAT_SVGZ, "svgz:svg", 1, NULL, &device_features_svgz},
 #endif
+    {FORMAT_SVG_INLINE, "svg_inline:svg", 1, NULL, &device_features_svg},
     {0, NULL, 0, NULL, NULL}
 };
