@@ -13,10 +13,11 @@
 #include <gvc/gvplugin_device.h>
 
 #include "gvplugin_quartz.h"
+#include <stdbool.h>
 
 #if __ENVIRONMENT_MAC_OS_X_VERSION_MIN_REQUIRED__ >= 1040 && defined(HAVE_PANGOCAIRO)
 
-const void *memory_data_consumer_get_byte_pointer(void *info)
+static const void *memory_data_consumer_get_byte_pointer(void *info)
 {
 	return info;
 }
@@ -34,7 +35,9 @@ static void quartz_format(GVJ_t *job)
 	/* image destination -> data consumer -> job's gvdevice */
 	/* data provider <- job's imagedata */
 	CGDataConsumerRef data_consumer = CGDataConsumerCreate(job, &device_data_consumer_callbacks);
-	CGImageDestinationRef image_destination = CGImageDestinationCreateWithDataConsumer(data_consumer, format_to_uti(job->device.id), 1, NULL);
+	CGImageDestinationRef image_destination =
+	  CGImageDestinationCreateWithDataConsumer(data_consumer,
+	    format_to_uti((format_type)job->device.id), 1, NULL);
 	CGDataProviderRef data_provider = CGDataProviderCreateDirect(job->imagedata, BYTES_PER_PIXEL * job->width * job->height, &memory_data_provider_callbacks);
 	
 	/* add the bitmap image to the destination and save it */
@@ -49,7 +52,7 @@ static void quartz_format(GVJ_t *job)
 		kCGImageAlphaPremultipliedFirst|kCGBitmapByteOrder32Little,	/* bitmap info: corresponds to CAIRO_FORMAT_ARGB32 */
 		data_provider,						/* data provider: from imagedata */
 		NULL,								/* decode: don't remap colors */
-		FALSE,								/* don't interpolate */
+		false, // don't interpolate
 		kCGRenderingIntentDefault			/* rendering intent (what to do with out-of-gamut colors): default */
 	);
 	CGImageDestinationAddImage(image_destination, image, NULL);
